/* $Id$

   Single Button Launcher - launch applications using a single hardware
   button.
   Copyright (C) 2001 Fraser McCrossan http://www.joat.ca

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
*/

#include <PalmOS.h>

#include "resource.h" /* resource IDs first */
#include "sbl.h"

UInt16 current_app; /* which app is the current scan position */
static UInt32 last_ticks; /* ticks value the last time the slot was changed */
Boolean override_buttons; /* whether to override the hardware buttons */
Boolean done_light; /* whether we have turned on the backlight already */
Boolean HwrBacklight(Boolean set, Boolean newState) SYS_TRAP(sysTrapHwrBacklightV33);

void SetAppSlot(UInt16 i, Char *label)
{
  FormType *form = FrmGetActiveForm();

  CtlSetLabel(FrmGetObjectPtr(form, FrmGetObjectIndex(form, i)),
	      (label[0] == '\0' ? "(Unassigned)" : label));
}

void SelectCurrentApp(Boolean draw)
{
  FormType *form = FrmGetActiveForm();
  RectangleType bounds;

  FrmGetObjectBounds(form, FrmGetObjectIndex(form, current_app + FirstApp),
		     &bounds);
  bounds.topLeft.x -= 2;
  bounds.topLeft.y -= 2;
  bounds.extent.x += 4;
  bounds.extent.y += 4;
  if (draw) {
    WinInvertRectangle(&bounds, 2);
    /* WinDrawRectangleFrame(rectangleFrame, &bounds); */
  }
  else {
    WinInvertRectangle(&bounds, 2);
    /* WinEraseRectangleFrame(rectangleFrame, &bounds); */
  }
  /* bounds.topLeft.x -= 1;
  bounds.topLeft.y -= 1;
  bounds.extent.x += 2;
  bounds.extent.y += 2;
  if (draw) {
    WinInvertRectangle(&bounds, 2);
    WinDrawRectangleFrame(rectangleFrame, &bounds);
  }
  else {
    WinInvertRectangle(&bounds, 2);
    WinEraseRectangleFrame(rectangleFrame, &bounds);
    } */
}

void ConfigMode(Boolean on)
{
  FormType *form = FrmGetActiveForm();
  UInt16 i;

  for (i = FirstApp; i <= LastApp; ++i) {
    CtlSetEnabled(FrmGetObjectPtr(form, FrmGetObjectIndex(form, i)), on);
  }
  /* perhaps erase the current slot highlight */
  if (on) {
    current_app = MaxApps;
  }
  /* set the labels appropriately */
  if (on) {
    FrmHideObject(form, FrmGetObjectIndex(form, LaunchLabel));
    FrmHideObject(form, FrmGetObjectIndex(form, WaitLabel));
    FrmShowObject(form, FrmGetObjectIndex(form, ConfigLabel));
  }
  else {
    FrmHideObject(form, FrmGetObjectIndex(form, ConfigLabel));
    FrmShowObject(form, FrmGetObjectIndex(form, LaunchLabel));
    FrmShowObject(form, FrmGetObjectIndex(form, WaitLabel));
  }
  override_buttons = !on;
}

/* how many buttons are depressed? */
UInt8 ButtonCount()
{
  UInt32 keystate = KeyCurrentState();
  UInt8 rv = 0;

  if (keystate & keyBitHard1) ++rv;
  if (keystate & keyBitHard2) ++rv;
  if (keystate & keyBitHard3) ++rv;
  if (keystate & keyBitHard4) ++rv;
  return rv;
}

void FindFirstButton()
{
  for (current_app = 0;
       current_app < MaxApps && prefs.apps[prefs.lastButtonPressed][current_app][0] == '\0';
       ++current_app);
}

/* if no keys are down, launch the current application, else check if we
   need to advance to the next slot */
void NextSlotOrLaunch()
{
  LocalID dbid;
  UInt8 bc;
  UInt32 delay;

  if (current_app >= MaxApps) {
    return;
  }
  /* check if keys are down */
  if ((bc = ButtonCount()) > 0) {
    /* double delay for the first slot */
    delay = (current_app > 0) ? DEFAULTDELAY : DEFAULTDELAY*2;
     if (current_app < MaxApps &&
	TimGetTicks() - last_ticks >= delay) {
      SelectCurrentApp(false);
      while (++current_app < MaxApps && prefs.apps[prefs.lastButtonPressed][current_app][0] == '\0');
      last_ticks = TimGetTicks();
      if (current_app < MaxApps) {
	SelectCurrentApp(true);
      }
      else {
	if (done_light) {
	  ConfigMode(true);
	  HwrBacklight(true, false);
	}
	else {
	  HwrBacklight(true, true);
	  FindFirstButton();
	  SysTaskDelay((Int32)DEFAULTDELAY);
	  SelectCurrentApp(true);
	  last_ticks = TimGetTicks();
	  done_light = true;
	}
      }
    }
  }
  else {
    /* find the application */
    if ((dbid = DmFindDatabase(0, prefs.apps[prefs.lastButtonPressed][current_app])) == 0) {
      (void) FrmCustomAlert(NotFoundError, prefs.apps[prefs.lastButtonPressed][current_app], "", "");
      ConfigMode(true); /* don't play any more */
    }
    else {
      if (SysUIAppSwitch(0, dbid, sysAppLaunchCmdNormalLaunch, NULL)
	  != errNone) {
	(void) FrmCustomAlert(CannotLaunchError, prefs.apps[prefs.lastButtonPressed][current_app],
			      "", "");
	ConfigMode(true); /* don't play any more */
      }
    }
  }
}

Boolean MainFormHandler(EventType *e)
{
  FormType *form = FrmGetActiveForm();
  UInt16 i;

  switch (e->eType) {

  case frmOpenEvent:
    FrmDrawForm(form);
    done_light = false;
    for (i = FirstApp; i <= LastApp; ++i) {
      SetAppSlot(i, prefs.apps[prefs.lastButtonPressed][i - FirstApp]);
    }
    if (current_app < MaxApps) {
      ConfigMode(false);
      last_ticks = TimGetTicks();
      SelectCurrentApp(true);
    }
    else {
      ConfigMode(true); /* this could happen if no buttons are assigned,
			   since the main routine will have set current_app */
    }
    return false;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case FirstApp:
    case SecondApp:
    case ThirdApp:
    case FourthApp:
    case FifthApp:
      if (PickForm_Run(prefs.apps[prefs.lastButtonPressed][e->data.ctlSelect.controlID - FirstApp])) {
	SetAppSlot(e->data.ctlSelect.controlID,
		   prefs.apps[prefs.lastButtonPressed][e->data.ctlSelect.controlID - FirstApp]);
      }
      return true;

    default:
      return false;
    }
    return false;

  case keyDownEvent:
    NextSlotOrLaunch();
    switch (e->data.keyDown.chr) {
    /* swallow key events for the four application buttons - nothing else,
       lest everything stops working */
    case vchrHard1:
    case vchrHard2:
    case vchrHard3:
    case vchrHard4:
      return override_buttons; /* only return true if we are overriding the
				  buttons; else return false; not that normally
				  this would immediately start a new
				  application, except that in our case it would
				  start *this* application with a different
				  launch code, so nothing actually happens */
      break;

    default:
      return false;
      break;

    }
    return false;

  case nilEvent:
    /* this is generated when the main event loop times out, and
       indicates that we should go to the next application; it's also
       generated at other times, so ignore ones below the timeout */
    /* first, though, check if no buttons are down, and if not, launch the
       current app */
    
    NextSlotOrLaunch();
    return false;

  default:
    return false;
  }

  return false;
}
