/* $Id$

   Beam Backup - backup vital data to another PalmOS handheld.
   Copyright (C) 2001 Fraser McCrossan http://www.joat.ca

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
*/

#include <PalmOS.h>

#include "resource.h" /* resource IDs first */
#include "sbl.h"

app_list all_apps;
extern Char *tname;

/* drawing callback function */
void DrawListItem(Int16 itemNum, RectangleType *bounds, Char **itemsText)
{
  app_entry *tdb;

  if (itemNum < all_apps.num_apps) {
    tdb = MemHandleLock(all_apps.apps);
    WinEraseRectangle(bounds, 0);
    WinDrawChars(tdb[itemNum].name, StrLen(tdb[itemNum].name),
		 bounds->topLeft.x, bounds->topLeft.y);
    MemHandleUnlock(all_apps.apps);
  }
}

Int16 App_Compare(void *r1, void *r2, Int32 other)
{
  return StrCaselessCompare(((app_entry *)r1)->name, ((app_entry *)r2)->name);
}

/* add a DB to the db list */
Err Add_db(app_list *list, UInt16 cardno, LocalID dbid, Char *name)
{
  Err e;
  app_entry *tdb;

  if (list->num_apps == 0) {
    if ((list->apps = MemHandleNew(sizeof(app_entry))) == 0) {
      return memErrNotEnoughSpace;
    }
  }
  else {
    if ((e = MemHandleResize(list->apps, sizeof(app_entry)*(list->num_apps+1)))
	!= 0) {
      return memErrNotEnoughSpace;
    }
  }
  /* if we got here, the memory allocation worked */
  tdb = MemHandleLock(list->apps);
  StrNCopy(tdb[list->num_apps].name, name, dmDBNameLength);
  MemHandleUnlock(list->apps);
  ++(list->num_apps);
  return errNone;
}

/* find all applications */
void GetAllApps()
{
  Err e;
  DmSearchStateType sst;
  UInt16 cardno;
  LocalID dbid;
  app_entry *tdb;
  Char name[dmDBNameLength];

  all_apps.num_apps = 0;
  e = DmGetNextDatabaseByTypeCreator(true /* new search */,
				     &sst, sysFileTApplication, 0, true,
				     &cardno, &dbid);
  while (e == errNone) {
    DmDatabaseInfo(cardno, dbid, name, NULL,
		   NULL, NULL, NULL, NULL,
		   NULL, NULL, NULL, NULL,
		   NULL);
    if (Add_db(&all_apps, cardno, dbid, name)
	!= errNone) {
      FrmAlert(MemoryError);
      return; /* if that didn't work, we better bug out */
    }
    e = DmGetNextDatabaseByTypeCreator(false /* new search */,
				       &sst, sysFileTApplication, 0, true,
				       &cardno, &dbid);
  }
  if (e != dmErrCantFind) {
    FrmCustomAlert(InternalError, "unable to find database",
		   __FUNCTION__, NULL);
  }
  if (all_apps.num_apps > 0) {
    tdb = MemHandleLock(all_apps.apps);
    SysQSort(tdb, (Int16) all_apps.num_apps, sizeof(*tdb), App_Compare, 0);
    MemHandleUnlock(all_apps.apps);
  }
}

void ClearAllApps()
{
  if (all_apps.apps) {
    MemHandleFree(all_apps.apps);
    all_apps.apps = NULL;
  }
  all_apps.num_apps = 0;
}

/* display the form, allow the user to pick a backup database from the list,
   return true (and the values in cardno and id) if they click OK, else
   return false */
Boolean PickForm_Run(Char *name)
{
  FormPtr prevFrm = FrmGetActiveForm();
  FormPtr form = FrmInitForm(PickForm);
  ListType *lst = FrmGetObjectPtr(form, FrmGetObjectIndex(form, PickList));
  app_entry *tdb;
  UInt16 result;
  Int16 selection;
  Boolean rv;

  GetAllApps();
  LstSetDrawFunction(lst, DrawListItem);
  LstSetListChoices(lst, NULL, all_apps.num_apps);
  LstSetSelection(lst, -1); /* no selection */
  result = FrmDoDialog(form);
  rv = false; /* by default, do nothing */
  if (result == Pick_Pick &&
      (selection = LstGetSelection(FrmGetObjectPtr(form, FrmGetObjectIndex(form, PickList)))) != noListSelection) {
    rv = true;
    tdb = MemHandleLock(all_apps.apps);
    StrNCopy(name, tdb[selection].name, dmDBNameLength);
    MemHandleUnlock(all_apps.apps);
  }
  ClearAllApps();
  FrmDeleteForm(form);
  FrmSetActiveForm(prevFrm);

  return rv;
}
